
# q-expansion of newform 352.2.m.e, downloaded from the LMFDB on 26 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 352
weight = 2
poly_data = [25, -175, 510, -350, 431, 36, 99, -43, 39, -11, 11, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[-68957262745775, 116322382813330, -282417040528075, -33033968740543, -50181676922288, 31959318188308, -22235847142081, 5089841799003, -6500442898522, 1158393636547, -689411783229, 12925652272], 206669284189945], [[-872744346462385, 617459400892385, -1496069668327950, -189418812627521, -255345496994316, 96085945450136, -116517829857207, 24891059999561, -33812071962329, 6710004699384, -3538073194043, 171694051509], 206669284189945], [[-79252706903350, 557460395900970, -180685886350265, 555127345742466, 78863770949376, 88370770295969, -29326445729798, 37072036936079, -6273835566151, 10879795136331, -1619787283007, 1093010781921], 206669284189945], [[79252706903350, -350791111711025, 180685886350265, -555127345742466, -78863770949376, -88370770295969, 29326445729798, -37072036936079, 6273835566151, -10879795136331, 1619787283007, -1093010781921], 206669284189945], [[-68337530631950, 481004938518940, -490703941148390, 489284366746582, 48983833899492, 102821973475408, -48099817253426, 39249052015903, -12495005400282, 12479616226722, -2450405323424, 1185075482262], 206669284189945], [[2642224095290, 903381683743390, -467441062100718, 1227122861994310, 201228017585416, 222516804049096, -78291500671051, 94111542385560, -17588897251336, 27618108154634, -4281926968294, 2733501225278], 206669284189945], [[-366378456407095, 1124311119485735, -998435647181393, 1138641532814873, 131257776542224, 250834913331188, -113516650123730, 101072886984887, -29182801971594, 29651783824912, -5503655367390, 2758290509831], 206669284189945], [[203977836612425, -1436069334478075, 554410550904434, -1445180437963130, -202494668236793, -284514273607468, 99242618937683, -117360387203075, 23991395901143, -33251403754467, 5247205770347, -3170108276134], 206669284189945], [[7410594838995, 3311104547261585, -1169321120091979, 3108320799340980, 473173927098043, 613573282631388, -194093475374298, 251104164538245, -48386040858223, 70986300515617, -11030588267832, 6759751764174], 206669284189945], [[1324968684082510, -3801356284840875, 2367534958049804, -3007480968446996, -220880731173839, -698793852941352, 300729036546526, -275899334623899, 76603457694164, -76812432137873, 13776035137694, -6923728370080], 206669284189945], [[-7554307496840, -2110762126897310, 1205623397343828, -3185417947781235, -563812477235096, -554905942184836, 183243705133141, -237405738244780, 41778451743806, -70862768140539, 10864290641274, -7100336146293], 206669284189945]]

hecke_ring_character_values = [[287, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [133, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [321, [-1, 0, 0, 0, 0, 0, 1, -1, 1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 1, 0, 0, -1, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0], [-1, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1], [1, 1, -1, 0, -1, -1, 0, 0, 1, -1, 0, 0], [0, 2, 0, 0, -2, 0, 0, 0, 0, 0, -1, 0], [1, 0, 0, 0, 0, 0, 1, 0, -1, 0, 0, 1], [0, 0, -1, 2, 0, -1, 0, 0, 0, 0, -1, 0], [2, 0, 2, 0, 0, 0, -2, 2, 0, 0, 0, 0], [-2, 0, 1, 0, 2, 0, 2, -2, 2, 1, 1, 1], [0, 0, 1, 1, 1, -1, -1, 3, -1, 0, 0, -1], [0, 2, -1, 2, 2, 0, 0, 2, -2, -1, 1, 1], [-1, 0, 1, -2, 0, -2, 0, -1, 3, 0, 1, 0], [0, 2, -1, 0, 0, 0, 3, -3, 0, -2, -2, 0], [5, 0, -1, 0, 0, -1, 0, 5, -6, -1, 0, 1], [0, 0, 2, 2, 2, -2, 0, 4, 0, 0, -1, -2], [-4, 2, -1, 2, 1, 0, 4, 0, 0, -1, 0, 0], [0, -2, 0, -2, -2, 2, -4, 0, 0, -1, 0, -2], [7, -2, 0, 0, 2, 2, -5, 5, 0, 1, 1, 0], [-1, 1, 0, 5, 5, -1, -2, 0, 1, 2, 0, 1], [0, -2, -3, -2, 0, 0, 0, 1, -1, -3, 0, 0], [0, 4, 1, 1, -3, -1, -1, -5, -1, 0, 0, -1], [-6, -3, 0, -1, -1, 3, 4, 0, 6, -1, 0, 1], [0, -4, 3, 0, 4, 4, -3, 3, 0, 2, 2, 0], [0, 0, 0, 0, 0, 0, -1, -3, -1, 0, 2, 0], [0, -4, 0, -4, 0, 4, 0, 4, 0, 0, 1, 0], [-5, 0, 0, 0, -3, 0, 5, -6, 6, 0, 1, 1], [-4, 0, 2, 0, 0, -3, 0, -4, -1, 2, 0, -2], [4, -2, 0, 0, 2, 2, 0, 0, 0, 2, 2, 0], [3, 0, 2, -2, 0, -2, 0, 3, -2, 0, 2, 0], [-7, 1, 0, 1, 1, -1, 2, 0, 7, -2, 0, -3], [1, -2, 4, 0, 2, 2, 1, -1, 0, 5, 5, 0], [-5, 2, 0, 0, 0, -2, 10, 0, 5, 0, 0, 2], [5, -4, 0, -4, -3, 0, -5, 10, -10, 0, -1, -1], [-10, 2, 0, 2, 2, -2, 2, 0, 10, 1, 0, 4], [-7, 0, -1, 4, 0, 3, 0, -7, -2, -1, 0, 1], [0, 0, -4, 4, 0, -4, 0, 0, 6, 1, -5, -1], [0, -6, -4, -3, 3, 3, -2, 7, -2, 0, -1, 4], [0, 4, -3, 1, -3, -1, 1, -7, 1, 0, 0, 3], [4, 0, 0, 0, 0, -2, 0, 4, -4, 1, -1, -1], [6, 0, 3, -2, 0, 3, 0, 6, -8, 0, 3, 0], [8, 0, 0, 0, 0, 0, -4, 0, -8, 1, 0, 0], [1, -8, -6, -8, -3, 0, -1, 0, 0, -6, 1, 1], [12, 0, 0, -6, -6, 0, -4, 0, -12, 1, 0, -2], [8, 4, 4, 0, 2, 2, 2, -2, 0, 0, 0, 0], [10, -4, 0, 0, 4, 4, -4, 4, 0, 2, 2, 0], [0, 2, 1, 3, 1, -3, 4, -4, 4, 0, -5, -1], [-1, 0, 7, 0, 0, 3, 0, -1, -12, 5, 2, -5], [-6, 2, 3, 2, 1, 0, 6, -4, 4, 3, -4, -4], [0, -6, -4, -2, 4, 2, 6, 0, 6, 0, -1, 4], [0, 0, 2, 2, 2, -2, -9, 1, -9, 0, -4, -2], [-1, 0, -3, 0, 0, -5, 0, -1, 4, 1, -4, -1], [-13, -4, -4, 0, 4, 4, 5, -5, 0, -1, -1, 0], [0, 4, 3, 1, -3, -1, -9, 5, -9, 0, 2, -3], [0, -8, -5, -8, -6, 0, 0, -1, 1, -5, 4, 4], [8, 8, 4, 0, -8, -8, -6, 6, 0, 0, 0, 0], [2, -4, 0, 6, 6, 4, -8, 0, -2, -1, 0, 2], [3, 4, 2, 4, -3, 0, -3, 8, -8, 2, 1, 1], [0, -2, 6, -2, 0, 2, 6, -2, 6, 0, -1, -6], [-3, 8, 0, 0, 0, -8, 8, 0, 3, -4, 0, 0], [-7, 0, -5, -4, 0, -1, 0, -7, 2, -3, -2, 3], [-12, 0, -1, 0, 2, 0, 12, -8, 8, -1, -3, -3], [4, -2, -5, 0, -4, -4, 3, -3, 0, -4, -4, 0], [-1, 0, -1, -8, 0, -1, 0, -1, 0, -3, 2, 3], [-3, 0, 0, 4, 4, 0, 13, 0, 3, -4, 0, -3], [0, -10, -2, -6, 4, 6, -10, 2, -10, 0, 3, 2], [8, 10, 1, 0, -4, -4, -5, 5, 0, 2, 2, 0], [3, 2, 2, 2, 4, 0, -3, 14, -14, 2, -2, -2], [-2, -1, 0, 5, 5, 1, 9, 0, 2, -3, 0, 4], [-4, 0, 8, 0, 0, 8, 0, -4, -2, 1, 7, -1], [-9, 2, -6, 0, 4, 4, -3, 3, 0, -3, -3, 0], [1, -4, 0, -4, -3, 0, -1, 4, -4, 0, 3, 3], [-5, 4, 4, 4, 1, 0, 5, -6, 6, 4, 3, 3], [8, 2, -4, 0, 0, 0, -4, 4, 0, -4, -4, 0], [-2, -3, 0, 3, 3, 3, 0, 0, 2, 7, 0, -3], [0, -4, 3, -3, 1, 3, -9, 9, -9, 0, 4, -3], [10, 0, 3, 0, -4, 0, -10, 2, -2, 3, -3, -3], [-4, 0, 2, 0, -2, -2, 12, -12, 0, -4, -4, 0], [-3, -4, 0, -6, -6, 4, -6, 0, 3, 0, 0, -4], [0, -8, -8, -2, 6, 2, -8, 10, -8, 0, -1, 8], [-8, 2, 1, 0, -4, -4, 5, -5, 0, -4, -4, 0], [4, 0, 4, 6, 0, -2, 0, 4, -10, 3, 1, -3], [0, -4, -7, 1, 5, -1, 11, -3, 11, 0, -2, 7], [-9, 2, 4, 2, -6, 0, 9, -2, 2, 4, -6, -6], [-6, -8, -4, 0, 0, 0, 8, -8, 0, -4, -4, 0], [0, 0, -2, 12, 0, -3, 0, 0, 11, 2, -4, -2], [0, 4, -1, 2, -2, -2, 7, -12, 7, 0, 0, 1], [-1, -2, 0, 0, 0, 2, -2, 0, 1, 8, 0, -6], [12, 4, -2, 0, -6, -6, 4, -4, 0, 0, 0, 0], [12, 4, 6, 0, -4, -4, 4, -4, 0, 4, 4, 0], [5, 5, 0, 1, 1, -5, 8, 0, -5, -2, 0, -3], [1, -3, 0, 9, 9, 3, 0, 0, -1, 0, 0, -1], [-13, 0, -3, -4, 0, 7, 0, -13, 4, -1, -2, 1], [-11, 0, 2, 0, 3, 0, 11, -17, 17, 2, 4, 4], [13, -4, -2, -4, 3, 0, -13, 5, -5, -2, 4, 4], [19, 0, 3, 8, 0, -5, 0, 19, -12, 3, 0, -3], [4, 0, -6, 10, 0, 0, 0, 4, 10, -5, -1, 5], [3, 10, 0, 10, 2, 0, -3, -6, 6, 0, -2, -2], [14, -1, 0, 1, 1, 1, -11, 0, -14, 1, 0, 2], [0, -6, -2, -10, -4, 10, 6, -14, 6, 0, -1, 2], [6, 0, -7, -2, 0, -1, 0, 6, 0, -2, -5, 2], [6, 8, -1, 8, -8, 0, -6, 6, -6, -1, -3, -3], [0, 2, -3, 3, 1, -3, -4, 20, -4, 0, 1, 3], [-11, 0, -2, 4, 0, -2, 0, -11, 16, 0, -2, 0], [-16, 0, 10, 0, 4, 4, 16, -16, 0, 6, 6, 0], [1, -4, 0, -2, -2, 4, -7, 0, -1, -4, 0, 7], [-6, -6, 1, -6, 13, 0, 6, -4, 4, 1, 8, 8], [-7, 4, -10, 0, -6, -6, -3, 3, 0, -5, -5, 0], [9, 1, 0, -7, -7, -1, 10, 0, -9, -4, 0, -1], [-22, 4, -1, 4, 4, 0, 22, -9, 9, -1, -2, -2], [0, -16, 3, -11, 5, 11, -3, 5, -3, 0, 4, -3], [-6, 0, 4, -2, 0, -10, 0, -6, 2, 1, 3, -1], [5, 0, -2, 0, -2, -2, 1, -1, 0, -5, -5, 0], [-4, 0, 9, 2, 0, 3, 0, -4, 4, 0, 9, 0], [7, -4, -8, -4, 1, 0, -7, -10, 10, -8, -3, -3], [3, 0, 5, 8, 0, 6, 0, 3, -15, 6, -1, -6], [-16, -3, 0, -1, -1, 3, 0, 0, 16, -3, 0, -5], [0, 4, -7, 5, 1, -5, 3, -13, 3, 0, 2, 7], [6, -2, -5, -2, -8, 0, -6, 10, -10, -5, -5, -5], [-1, 6, 8, 0, -6, -6, -9, 9, 0, 3, 3, 0], [0, -10, -8, 0, 2, 2, -8, 8, 0, -6, -6, 0], [0, 6, 7, -4, -10, 4, -3, 0, -3, 0, 6, -7], [2, -8, 0, -4, -4, 8, -26, 0, -2, -1, 0, -6], [-12, -8, -8, 0, 0, 0, -4, 4, 0, 0, 0, 0], [0, 2, 4, 1, -1, -1, 20, -5, 20, 0, -9, -4], [-2, 0, 6, -10, 0, 8, 0, -2, -6, 5, 1, -5], [-8, -10, 0, -2, -2, 10, 6, 0, 8, 3, 0, -4], [21, 4, 0, 4, -3, 0, -21, 10, -10, 0, 3, 3], [20, -12, -14, 0, 0, 0, 6, -6, 0, -8, -8, 0], [22, -10, 3, -10, 4, 0, -22, 16, -16, 3, -3, -3], [0, -2, 2, -3, -1, 3, -22, 27, -22, 0, -1, -2], [11, 5, 0, -3, -3, -5, 4, 0, -11, 2, 0, -9], [-15, 0, -1, 4, 0, -5, 0, -15, 20, 3, -4, -3], [0, 0, 2, -2, -2, 2, 6, 14, 6, 0, 5, -2], [0, -2, 1, -6, -4, 6, -17, -2, -17, 0, 2, -1], [8, -2, 2, 0, -8, -8, -6, 6, 0, -4, -4, 0], [-12, 0, -10, 2, 0, -10, 0, -12, 12, -5, -5, 5], [24, -5, 0, 1, 1, 5, -21, 0, -24, 3, 0, -4], [8, -6, 0, -4, -4, 6, -18, 0, -8, 3, 0, -6], [-5, -4, 0, -14, -14, 4, -5, 0, 5, -6, 0, 1], [-4, 0, 10, -4, 0, 9, 0, -4, -1, 2, 8, -2], [-20, 6, -5, 6, 6, 0, 20, -6, 6, -5, 5, 5], [0, 4, -3, -7, -11, 7, 1, 7, 1, 0, 4, 3], [0, -2, 6, -11, -9, 11, -2, -1, -2, 0, -3, -6], [2, -4, -9, -4, -4, 0, -2, 8, -8, -9, 3, 3], [-3, 0, 7, -4, 0, 15, 0, -3, 8, 9, -2, -9], [-8, 0, 0, 2, 2, 0, 26, 0, 8, -7, 0, 4], [18, -6, -11, 0, -2, -2, -17, 17, 0, -12, -12, 0], [3, 10, -2, 0, -10, -10, 11, -11, 0, -3, -3, 0], [-9, -3, 0, 5, 5, 3, 26, 0, 9, 0, 0, -3], [18, 0, -10, -6, 0, -10, 0, 18, 4, -7, -3, 7], [-14, 10, 7, 0, -6, -6, 17, -17, 0, 4, 4, 0], [-18, 0, -10, 8, 0, -3, 0, -18, 5, -8, -2, 8], [-17, 4, 0, 4, 9, 0, 17, -16, 16, 0, -1, -1], [-24, 11, 0, 1, 1, -11, 19, 0, 24, -1, 0, 2], [0, -8, -1, -11, -3, 11, -11, -3, -11, 0, 2, 1], [17, 5, 0, -7, -7, -5, -40, 0, -17, -6, 0, 3], [-5, 4, 0, 2, 2, -4, -13, 0, 5, 2, 0, 5], [0, 2, 3, -2, -4, 2, -3, 10, -3, 0, 8, -3], [8, 14, 0, 16, 16, -14, -6, 0, -8, 7, 0, 2], [-13, 14, -2, 0, -10, -10, 7, -7, 0, -3, -3, 0], [-2, 0, -1, 0, -4, 0, 2, -15, 15, -1, 4, 4], [-10, -4, 4, 0, -4, -4, 4, -4, 0, -6, -6, 0], [-7, 0, -3, 4, 0, -9, 0, -7, 10, 5, -8, -5], [0, 6, -4, 12, 6, -12, 14, -6, 14, 0, -7, 4], [-9, -8, 2, -8, 1, 0, 9, 0, 0, 2, 7, 7], [18, 4, 4, 0, -8, -8, -10, 10, 0, 2, 2, 0], [-16, 8, 1, 8, -8, 0, 16, -16, 16, 1, 1, 1]]
